
/*
 *
 * Copyright (C) Eicon Technology Corporation, 2000.
 *
 * This source file is supplied for the exclusive use with Eicon
 * Technology Corporation's range of DIVA Server Adapters.
 *
 * Eicon File Revision :    1.0  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY OF ANY KIND WHATSOEVER INCLUDING ANY 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


static char * cau_1tr6[] = {
  "0",
  "Invalid call reference",
  "2",
  "Bearer service not implemented",
  "4",
  "5",
  "Channel unacceptable",
  "Call identity does not exist",
  "Call identity in use",
  "9",
  "No channel available",
  "11",
  "12",
  "13",
  "14",
  "15",
  "Requested facility not implemented",
  "Requested facility not subscribed",
  "18",
  "19",
  "20",
  "21",
  "22",
  "23",
  "24",
  "25",
  "26",
  "27",
  "28",
  "29",
  "30",
  "31",
  "Outgoing call barred",
  "User access busy",
  "Wrong closed user group",
  "35",
  "36",
  "Semipermanent connection not allowed",
  "38",
  "39",
  "40",
  "41",
  "42",
  "43",
  "44",
  "45",
  "46",
  "47",
  "Reverse charging not allowed at originating end",
  "49",
  "50",
  "51",
  "52",
  "Destination not obtainable",
  "54",
  "55",
  "Number changed",
  "Out of order",
  "No User responding",
  "User busy",
  "60",
  "Incoming calls barred",
  "Call rejected",
  "63",
  "64",
  "65",
  "66",
  "67",
  "68",
  "69",
  "70",
  "71",
  "72",
  "73",
  "74",
  "75",
  "76",
  "77",
  "78",
  "79",
  "80",
  "81",
  "82",
  "83",
  "84",
  "85",
  "86",
  "87",
  "Incompatible destination",
  "Network congestion",
  "Remote user initiated",
  "91",
  "92",
  "93",
  "94",
  "95",
  "96",
  "97",
  "98",
  "99",
  "100",
  "101",
  "102",
  "103",
  "104",
  "105",
  "106",
  "107",
  "108",
  "109",
  "110",
  "111",
  "Local procedure error",
  "Remote procedure error",
  "Remote user suspended",
  "Remote user resumed",
  "116",
  "117",
  "118",
  "119",
  "120",
  "121",
  "122",
  "123",
  "124",
  "125",
  "126",
  "User info discarded locally"
};
