
struct protos {
	unsigned int val;
	char *name;
};

struct protos protonames[] = {
 { 0x0001,"Padding Protocol" },
 { 0x0021,"Internet Protocol" },
 { 0x0023,"OSI Network Layer" },
 { 0x0025,"Xerox NS IDP" },
 { 0x0027,"DECnet Phase IV" },
 { 0x0029,"Appletalk" },
 { 0x002b,"Novell IPX" },
 { 0x002d,"Van Jacobson Compressed TCP/IP" },
 { 0x002f,"Van Jacobson Uncompressed TCP/IP" },
 { 0x0031,"Bridging PDU" },
 { 0x0033,"Stream Protocol (ST-II)" },
 { 0x0035,"Banyan Vines" },
 { 0x0037,"reserved (until 1993)" },
 { 0x0039,"AppleTalk EDDP" },
 { 0x003b,"AppleTalk SmartBuffered" },
 { 0x003d,"Multi-Link [RFC1717]" },
 { 0x003f,"NETBIOS Framing" },
 { 0x0041,"Cisco Systems" },
 { 0x0043,"Ascom Timeplex" },
 { 0x0045,"Fujitsu Link Backup and Load Balancing (LBLB)" },
 { 0x0047,"DCA Remote Lan" },
 { 0x0049,"Serial Data Transport Protocol (PPP-SDTP)" },
 { 0x004b,"SNA over 802.2" },
 { 0x004d,"SNA" },
 { 0x004f,"IP6 Header Compression" },
 { 0x0051,"KNX Bridging Data" },
 { 0x0053,"Encryption" },
 { 0x0055,"Individual Link Encryption" },
 { 0x006f,"Stampede Bridging" },
 { 0x00fb,"compression on single link in multilink group" },
 { 0x00fd,"1st choice compression" },

 { 0x0201,"802.1d Hello Packets" },
 { 0x0203,"IBM Source Routing BPDU" },
 { 0x0205,"DEC LANBridge100 Spanning Tree" },
 { 0x0207,"Cisco Discovery Protocol" },
 { 0x0209,"Netcs Twin Routing" },
 { 0x0231,"Luxcom" },
 { 0x0233,"Sigma Network Systems" },
 { 0x0235,"Apple Client Server Protocol" },

 { 0x4001,"Cray Communications Control Protocol" },
 { 0x4003,"CDPD Mobile Network Registration Protocol" },
 { 0x4021,"Stacker LZS" },

 { 0x8021,"Internet Protocol Control Protocol" },
 { 0x8023,"OSI Network Layer Control Protocol" },
 { 0x8025,"Xerox NS IDP Control Protocol" },
 { 0x8027,"DECnet Phase IV Control Protocol" },
 { 0x8029,"Appletalk Control Protocol" },
 { 0x802b,"Novell IPX Control Protocol" },
 { 0x8031,"Bridging NCP" },
 { 0x8033,"Stream Protocol Control Protocol" },
 { 0x8035,"Banyan Vines Control Protocol" },
 { 0x803d,"Multi-Link Control Protocol" },
 { 0x803f,"NETBIOS Framing Control Protocol" },
 { 0x8041,"Cisco Systems Control Protocol" },
 { 0x8043,"Ascom Timeplex" },
 { 0x8045,"Fujitsu LBLB Control Protocol" },
 { 0x8047,"DCA Remote Lan Network Control Protocol (RLNCP)" },
 { 0x8049,"Serial Data Control Protocol (PPP-SDCP)" },
 { 0x804b,"SNA over 802.2 Control Protocol" },
 { 0x804d,"SNA Control Protocol" },
 { 0x804f,"IP6 Header Compression Control Protocol" },
 { 0x8051,"KNX Bridging Control Protocol" },
 { 0x8053,"Encryption Control Protocol" },
 { 0x8055,"Individual Link Encryption Control Protocol" },
 { 0x806f,"Stampede Bridging Control Protocol" },
 { 0x80fb,"compression on single link in multilink group control" },
 { 0x80fd,"Compression Control Protocol" },
 { 0x8207,"Cisco Discovery Protocol Control" },
 { 0x8209,"Netcs Twin Routing" },
 { 0x8235,"Apple Client Server Protocol Control" },

 { 0xc021,"Link Control Protocol" },
 { 0xc023,"Password Authentication Protocol" },
 { 0xc025,"Link Quality Report" },
 { 0xc027,"Shiva Password Authentication Protocol" },
 { 0xc029,"CallBack Control Protocol (CBCP)" },
 { 0xc02b,"BACP Bandwidth Allocation Control Protocol" },
 { 0xc081,"Container Control Protocol" },
 { 0xc223,"Challenge Handshake Authentication Protocol" },
 { 0xc225,"RSA Authentication Protocol" },
 { 0xc26f,"Stampede Bridging Authorization Protocol" },
 { 0xc281,"Proprietary Authentication Protocol" },
 { 0xc283,"Proprietary Authentication Protocol" },
 { 0xc481,"Proprietary Node ID Authentication Protocol" }
};



